@@ -9,29 +9,29 @@ module ChefZero
         response = super(request)
 
         if request.query_params['email']
-          results = FFI_Yajl::Parser.parse(response[2], :create_additions => false)
+          results = parse_json(response[2])
           new_results = {}
           results.each do |name, url|
             record = get_data(request, request.rest_path + [ name ], :nil)
             if record
-              record = FFI_Yajl::Parser.parse(record, :create_additions => false)
+              record = parse_json(record)
               new_results[name] = url if record['email'] == request.query_params['email']
             end
           end
-          response[2] = FFI_Yajl::Encoder.encode(new_results, :pretty => true)
+          response[2] = to_json(new_results)
         end
 
         if request.query_params['verbose']
-          results = FFI_Yajl::Parser.parse(response[2], :create_additions => false)
+          results = parse_json(response[2])
           results.each do |name, url|
             record = get_data(request, request.rest_path + [ name ], :nil)
             if record
-              record = FFI_Yajl::Parser.parse(record, :create_additions => false)
+              record = parse_json(record)
               record = ChefData::DataNormalizer.normalize_user(record, name, identity_keys, server.options[:osc_compat])
               results[name] = record
             end
           end
-          response[2] = FFI_Yajl::Encoder.encode(results, :pretty => true)
+          response[2] = to_json(results)
         end
         response
       end
@@ -39,21 +39,47 @@ module ChefZero
       def post(request)
         # First, find out if the user actually posted a public key.  If not, make
         # one.
-        request_body = FFI_Yajl::Parser.parse(request.body, :create_additions => false)
+        request_body = parse_json(request.body)
         public_key = request_body['public_key']
-        if !public_key
+
+        skip_key_create = !request.api_v0? && !request_body["create_key"]
+
+        if !public_key && !skip_key_create
           private_key, public_key = server.gen_key_pair
           request_body['public_key'] = public_key
-          request.body = FFI_Yajl::Encoder.encode(request_body, :pretty => true)
+          request.body = to_json(request_body)
+        elsif skip_key_create
+          request_body['public_key'] = nil
+          request.body = to_json(request_body)
         end
 
         result = super(request)
 
         if result[0] == 201
           # If we generated a key, stuff it in the response.
-          response = FFI_Yajl::Parser.parse(result[2], :create_additions => false)
-          response['private_key'] = private_key if private_key
-          response['public_key'] = public_key unless request.rest_path[0] == 'users'
+          user_data = parse_json(result[2])
+
+          key_data = {}
+          key_data['private_key'] = private_key if private_key
+          key_data['public_key'] = public_key unless request.rest_path[0] == 'users'
+
+          response =
+            if request.api_v0?
+              user_data.merge!(key_data)
+            elsif skip_key_create && !public_key
+              user_data
+            else
+              actor_name = request_body["name"] || request_body["username"] || request_body["clientname"]
+
+              relpath_to_default_key = [ actor_name, "keys", "default" ]
+              key_data["uri"] = build_uri(request.base_uri, request.rest_path + relpath_to_default_key)
+              key_data["public_key"] = public_key
+              key_data["name"] = "default"
+              key_data["expiration_date"] = "infinity"
+              user_data["chef_key"] = key_data
+              user_data
+            end
+
           json_response(201, response)
         else
           result
